#include <SoftwareSerial.h>
#define rxPin 3 
#define txPin 2 
SoftwareSerial esp01(txPin, rxPin); // SoftwareSerial NAME(TX, RX);

const String ssid_AP = "ESP01";    // your network SSID (name)
const String pass_AP = "12345678"; //"1234test";         // your network password

char ssid[21] = "skynet";    // "SK_WiFiGIGA40F7" your network SSID (name)
char pass[21] = "skynet00";  // "1712037218" your network password 

String ip = "";
String ap = "";

void ipCheck(String response) {
  int st, ed;
  if (response.indexOf(F("PIP")) != -1) {
    st = response.indexOf(F("PIP"))+5;
    ed = response.indexOf('"', st+1);
    ap = response.substring(st, ed);
    response.remove(0, ed+1);
    Serial.print(F("ap: ")); Serial.println(ap);
  }
  if (response.indexOf(F("AIP")) != -1) {
    st = response.indexOf(F("AIP"))+5;
    ed = response.indexOf('"', st+1);
    ip = response.substring(st, ed);
    Serial.print(F("ip: ")); Serial.println(ip);
  }
}

uint8_t retry = 1;

String sendData(String order) {
  String command = F("AT+");
  command += order;
  command += F("\r\n");
  esp01.print(command); // send the read character to the esp01
  unsigned long int time = millis();
  String response = "";
  while( (time+20000) > millis()) {
    while(esp01.available()) {  // The esp has data so display its output to the serial window
      char c = esp01.read(); // read the next character.
      response+=c;
    }
    if (response.indexOf(F("OK")) != -1) { retry = 0; Serial.println(F("OK")); break; } 
    else if (response.indexOf(F("FAIL")) != -1 || response.indexOf(F("ERROR")) != -1) { 
      if (response.indexOf(F("FAIL")) != -1) retry++;
      else if (response.indexOf(F("ERROR")) != -1) { Serial.println(F("ERROR")); break; } 
      Serial.print(command); Serial.println(F("Failed")); break; 
    }
  }
  return response;
}

String addID(String order, String ssid_AP, String pass_AP) {
  String temp = order+'"'+ssid_AP+'"'+','+'"'+pass_AP+'"';
  return temp;
}

void setup() {
  Serial.begin(9600);
  esp01.begin(9600); // your esp's baud rate might be different
  sendData(F("CWQIF")); // disconnect AP if connected
  sendData(F("CWQAP")); // disconnect wifi if connected
  sendData(F("CWMODE=3")); // configure as access point (working mode: AP+STA)
  sendData(F("CWDHCP=1,1"));
  sendData(addID(F("CWSAP="),ssid_AP,pass_AP)+','+String(10)+','+String(3)); 
  retry = 1;
  while(retry) { 
    sendData(addID(F("CWJAP="),ssid,pass)); 
    if (retry > 1) { Serial.print(F("Retrying to connect: ")); Serial.println(retry-1); }
    if (retry > 4) { Serial.println(F("connection failed")); retry = 0; }
  }
  sendData(F("CIPMUX=1")); // configure for multiple connections
  sendData(F("CIPSERVER=1,80")); // turn on server on port 80
  ipCheck(sendData(F("CIFSR"))); // // get ip address
}

String json_parser(String s, String a) { 
  String val;
  if (s.indexOf(a) != -1) {
    int st_index = s.indexOf(a);
    int val_index = s.indexOf(':', st_index);
    if (s.charAt(val_index + 1) == '"') {
      int ed_index = s.indexOf('"', val_index + 2);
      val = s.substring(val_index + 2, ed_index);
    }
    else {
      int ed_index = s.indexOf(',', val_index + 1);
      val = s.substring(val_index + 1, ed_index);
    }
  } 
  else {
    Serial.print(a); Serial.println(F(" is not available"));
  }
  return val;
}

// OPEN WEATHER MAP
// https://api.openweathermap.org/data/2.5/weather?q=Seoul,KR&APPID=YOUR_API_KEY
const char host[] PROGMEM = "api.openweathermap.org";
const char url[]  PROGMEM = "/data/2.5/weather?q=Seoul,KR&APPID=YOUR_API_KEY"; // YOUR_API_KEY 수정

bool OK200 = false;
bool sentREQ = false;
bool gotWeather = false;

void get_weather() {
  Serial.println(F("Starting connection to server..."));
  bool connectOK = false;
  String cip = F("AT+CIPSTART=3,"); // connection ID 3 (ID: 0 ~ 4)
  cip += '"'; cip += F("TCP"); cip += '"';
  cip += ','; cip += '"';
  cip += (const __FlashStringHelper *)host;
  cip += '"'; cip += ',';
  cip += F("80"); // port number
  cip += F("\r\n");
  esp01.print(cip);
  unsigned long int time = millis();
  while (time+2000 > millis()) {
    cip = esp01.readStringUntil('\n');
    if (cip.indexOf(F("OK")) != -1) { connectOK = true; Serial.println(F("OK")); break; }
    else if (cip.indexOf(F("ERROR")) != -1) { Serial.println(F("ERROR")); break; }
  }
  if (connectOK) {
    Serial.println(F("Connected to server"));
    String temp = F("GET ");
    temp += (const __FlashStringHelper *)url;
    temp += F(" HTTP/1.1\r\nHost: ");
    temp += (const __FlashStringHelper *)host;
    temp += F("\r\nConnection: close\r\n\r\n");
    cip = F("AT+CIPSEND=");
    cip += 3; // connection ID 3 (ID: 0 ~ 4)
    cip += ',';
    cip += temp.length();
    cip += F("\r\n");
    esp01.print(cip);
    time = millis();
    while (time+2000 > millis()) {
      cip = esp01.readStringUntil('\n');
      if (cip.indexOf(F("OK")) != -1) { Serial.println(F("OK")); break; }
      else if (cip.indexOf(F("ERROR")) != -1) { Serial.println(F("ERROR")); break; }
    }
    esp01.print(temp);
    sentREQ = true;
    gotWeather = true;
  }
}

// RSS 날씨 정보 저장 변수
String line = "";
uint8_t tryCount = 0;

void loop() {
  if (!ip.startsWith(F("0"))) { // 공유기로부터 IP주소를 할당받은 경우
    if (!gotWeather) {
      if (tryCount < 3) { get_weather(); tryCount++; }
    }
  }
  while (esp01.available()) { 
    line = esp01.readStringUntil('\n');
    Serial.println(line);
    if (sentREQ) {
      if (!OK200) {
        if (line.startsWith(F("+"))) {
          if (line.indexOf(F("200")) != -1) OK200 = true;
          else { OK200 = false; gotWeather = false; }
        }
      } else {
        if (line.startsWith(F("{"))) {
          Serial.println(line);
          Serial.println(F("weather data for parsing"));
          String description = json_parser(line, F("description"));
          Serial.print(F("description: ")); Serial.println(description);
          String pressure = json_parser(line, F("pressure"));
          Serial.print(F("pressure: ")); Serial.println(pressure);
          String wind = json_parser(line, F("speed"));
          Serial.print(F("wind: ")); Serial.println(wind);
          String Temp = json_parser(line, F("temp"));
          Serial.print(F("Temp: ")); Serial.println(Temp);
          line = "";
          gotWeather = true; // 날씨 작업 종료
          tryCount = 0; OK200 = false; sentREQ = false; // 초기화
        }
      }
    } else {
      Serial.println(line);
    }
  }
}
